
/* This file is part of the GMOD package */

#include <signal.h>
#include <stdio.h>
#include <sys/time.h>

#ifdef USE_LOCAL
#include "soundcard.h"
#else
#include <sys/soundcard.h>
#endif

#include <sys/types.h>
#include <unistd.h>

#ifdef USE_NCURSES
#include <ncurses.h>
#endif

#include "defines.h"
#include "structs.h"
#include "globals.h"
#include "protos.h"

unsigned long current_time;
static unsigned long real_time;
int position_change = 0;

unsigned long
get_real_time ()
{
  struct timeval tv;
  struct timezone tz =
  {0, DST_NONE};

  gettimeofday (&tv, &tz);
  return ((tv.tv_sec * 100) + (tv.tv_usec / 10000));
}

void
timer_handler (int sig)
{
  int keypress = EOF;
  unsigned short volume;
  int tmp_volume;
  static int sync_ticks, reset_arg = MY_FALSE;
  char message[80];
  static unsigned int keyed_arg = 0;

  signal (sig, timer_handler);

  switch (sig)
    {
    case SIGALRM:
      if (!background)
	{
#ifdef USE_NCURSES
	  keypress = getch ();
#else
	  keypress = getchar ();
#endif
	}
      current_time += 100;
      break;
    case SIGTERM:
      keypress = 'q';
      break;
    case SIGUSR1:
      keypress = 'p';
      break;
    case SIGUSR2:
      keypress = 'n';
      break;
    }

  if (sync_ticks++ >= 5)
    {
      sync_ticks = 0;
      current_time = get_real_time () - real_time;
    }

  do
    {
      get_message (message, current_time);
      if ((message[0] != '\0') && (background == 0))
	fprintf (stderr, "%s", message);
    }
  while (message[0] != '\0');

  while (keypress != EOF)
    {
      switch (keypress)
	{
#ifdef USE_NCURSES
	case KEY_LEFT:
#endif
	case 'b':
	  if (keyed_arg == 0)
	    position_change--;
	  else
	    position_change -= keyed_arg;
	  reset_arg = MY_TRUE;
	  break;
#ifdef USE_NCURSES
	case KEY_RIGHT:
#endif
	case 'f':
	  if (keyed_arg == 0)
	    position_change++;
	  else
	    position_change += keyed_arg;
	  reset_arg = MY_TRUE;
	  break;
#ifdef USE_NCURSES
	case KEY_NPAGE:
#endif
	case 'n':
	  stop_flag = STOP_NEXT;
	  dump_enabled = 0;
	  break;
#ifdef USE_NCURSES
	case KEY_PPAGE:
#endif
	case 'p':
	  stop_flag = STOP_PREV;
	  dump_enabled = 0;
	  break;
	case 'q':
	  stop_flag = STOP_EXIT;
	  dump_enabled = 0;
	  break;
#ifdef USE_NCURSES
	case KEY_UP:
#endif
	case '+':
	  if (mixerfd != -1)
	    {
	      ioctl (mixerfd, MIXER_READ (SOUND_MIXER_SYNTH), &volume);
	      volume &= 0xff;
	      if (keyed_arg == 0)
		tmp_volume = (int) volume + 1;
	      else
		tmp_volume = (int) volume + keyed_arg;
	      if (tmp_volume > MIXER_MAX_VOL)
		tmp_volume = MIXER_MAX_VOL;
	      volume = tmp_volume * 256 + tmp_volume;
	      ioctl (mixerfd, MIXER_WRITE (SOUND_MIXER_SYNTH), &volume);
	    }
	  reset_arg = MY_TRUE;
	  break;
#ifdef USE_NCURSES
	case KEY_DOWN:
#endif
	case '-':
	  if (mixerfd != -1)
	    {
	      ioctl (mixerfd, MIXER_READ (SOUND_MIXER_SYNTH), &volume);
	      volume &= 0xff;
	      if (keyed_arg == 0)
		tmp_volume = (int) volume - 1;
	      else
		tmp_volume = (int) volume - keyed_arg;
	      if (tmp_volume < MIXER_MIN_VOL)
		tmp_volume = MIXER_MIN_VOL;
	      volume = tmp_volume * 256 + tmp_volume;
	      ioctl (mixerfd, MIXER_WRITE (SOUND_MIXER_SYNTH), &volume);
	    }
	  reset_arg = MY_TRUE;
	  break;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	  if (reset_arg == MY_TRUE)
	    {
	      reset_arg = MY_FALSE;
	      keyed_arg = 0;
	    }
	  keyed_arg = (keyed_arg * 10) + (keypress - '0');
	  break;
	}

      if (!background)
	{
#ifdef USE_NCURSES
	  keypress = getch ();
#else
	  keypress = getchar ();
#endif
	}
      else
	keypress = EOF;
    }
}

void
timer_set (int mode)
{
  struct itimerval tvalue;
  char message[80];

  if (mode == TIMER_ON)
    {
      tvalue.it_interval.tv_sec = 1;
      tvalue.it_value.tv_sec = 1;
      signal (SIGALRM, timer_handler);
      signal (SIGTERM, timer_handler);
      signal (SIGUSR1, timer_handler);
      signal (SIGUSR2, timer_handler);

      current_time = 0;
      real_time = get_real_time ();
    }
  else
    {
      tvalue.it_interval.tv_sec = 0;
      tvalue.it_value.tv_sec = 0;
      signal (SIGALRM, SIG_IGN);
      signal (SIGTERM, SIG_DFL);
      signal (SIGUSR1, SIG_DFL);
      signal (SIGUSR2, SIG_DFL);

      do
	{
	  get_message (message, 0xffffffff);
	}
      while (message[0] != '\0');
    }

  tvalue.it_interval.tv_usec = 0;
  tvalue.it_value.tv_usec = 0;

  setitimer (ITIMER_REAL, &tvalue, NULL);
}
